/*
 * Copyright 2009 OW2 Chameleon
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.ow2.chameleon.json.test;

import java.text.ParseException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import junit.framework.Assert;
import junit.framework.TestCase;

import org.ow2.chameleon.json.impl.JSONServiceProvider;


public class ToMapTest extends TestCase {

    JSONServiceProvider svc;

    Map map_simple, map_simple_2, map_simple_3, collection, collection_set, collection_array;

    public void setUp() {
        svc = new JSONServiceProvider();

        map_simple = new HashMap();
        map_simple.put("foo", "foo");

        map_simple_2 = new HashMap();
        map_simple_2.put("foo", "foo");
        map_simple_2.put("bar", "bar");

        map_simple_3 = new HashMap();
        map_simple_3.put("foo", "foo");
        map_simple_3.put("one", new Integer(1));
        map_simple_3.put("true", Boolean.TRUE);
        map_simple_3.put("false", Boolean.FALSE);
        map_simple_3.put("nil", svc.getNull());
        map_simple_3.put("double", new Double(2.4));
        map_simple_3.put("object", map_simple_2);

        Map item1 = new HashMap();
        item1.put("x", new Integer(1));
        item1.put("y", new Integer(1));
        Map item2 = new HashMap();
        item2.put("x", new Integer(2));
        item2.put("y", new Integer(2));
        Map item3 = new HashMap();
        item3.put("x", new Integer(3));
        item3.put("y", new Integer(3));

        collection = new HashMap();
        List list = new ArrayList();
        list.add(item1);
        list.add(item2);
        list.add(item3);
        collection.put("items", list);


        collection_set = new HashMap();
        Set set = new HashSet();
        set.add(item1);
        set.add(item2);
        set.add(item3);
        collection_set.put("items", set);

        collection_array = new HashMap();
        Map[] array = new Map[] {
                item1, item2, item3
        };
        collection_array.put("items", array);

    }

    public void testToJSON() throws ParseException {
        String json = svc.toJSON(map_simple);
        Map m1 = svc.fromJSON(json);
        Assert.assertEquals("foo", m1.get("foo"));
        Assert.assertEquals(new Integer(1), new Integer(m1.size()));

        json = svc.toJSON(map_simple_2);
        Map m2 = svc.fromJSON(json);
        Assert.assertEquals("foo", m2.get("foo"));
        Assert.assertEquals("bar", m2.get("bar"));
        Assert.assertEquals(new Integer(2), new Integer(m2.size()));

        json = svc.toJSON(new HashMap());
        Map m3 = svc.fromJSON(json);
        Assert.assertEquals(new Integer(0), new Integer(m3.size()));

        json = svc.toJSON((Map) null);
        // This reutnrs null ... TODO
        Map m5 = svc.fromJSON(json);
        Assert.assertEquals(new Integer(0), new Integer(m5.size()));

        json = svc.toJSON(map_simple_3);
        Map m6 = svc.fromJSON(json);
        Assert.assertEquals(new Integer(1), m6.get("one"));
        Assert.assertEquals(new Double(2.4), m6.get("double"));
        Assert.assertEquals(Boolean.TRUE, m6.get("true"));
        Assert.assertEquals(Boolean.FALSE, m6.get("false"));
        Assert.assertEquals("foo", m6.get("foo"));
        Assert.assertEquals(svc.getNull(), m6.get("nil"));
        Assert.assertFalse(null == m6.get("nil")); // It's not null it's NULL (Javascript) !
        Map m62 = (Map) m6.get("object");
        Assert.assertEquals("foo", m62.get("foo"));
        Assert.assertEquals("bar", m62.get("bar"));
        Assert.assertEquals(new Integer(2), new Integer(m62.size()));
    }

    public void testFromJSONWithList() throws ParseException {
        String  json = svc.toJSON(collection);
        Map m1 = svc.fromJSON(json);
        Assert.assertEquals(new Integer(1), new Integer(m1.size()));
        List list = (List) m1.get("items");
        Assert.assertEquals(new Integer(3), new Integer(list.size()));
        Map m11 = (Map) list.get(0);
        Assert.assertEquals(new Integer(1), m11.get("x"));
        Assert.assertEquals(new Integer(1), m11.get("y"));
    }

    public void testFromJSONWithArray() throws ParseException {
        String  json = svc.toJSON(collection_array);
        Map m1 = svc.fromJSON(json);
        Assert.assertEquals(new Integer(1), new Integer(m1.size()));
        List list = (List) m1.get("items");
        Assert.assertEquals(new Integer(3), new Integer(list.size()));
        Map m11 = (Map) list.get(0);
        Assert.assertEquals(new Integer(1), m11.get("x"));
        Assert.assertEquals(new Integer(1), m11.get("y"));
    }

    public void testFromJSONWithSet() throws ParseException {
        String  json = svc.toJSON(collection_set);
        Map m1 = svc.fromJSON(json);
        Assert.assertEquals(new Integer(1), new Integer(m1.size()));
        List list = (List) m1.get("items");
        Assert.assertEquals(new Integer(3), new Integer(list.size()));
    }

}
