/*
 * Copyright 2009 OW2 Chameleon
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.ow2.chameleon.json.test;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.text.ParseException;
import java.util.List;
import java.util.Map;

import junit.framework.TestCase;

import junit.framework.Assert;
import org.ow2.chameleon.json.impl.JSONServiceProvider;


public class XMLTest extends TestCase {

    public static final String ROOT_STR = "src/test/resources";
    public static final File ROOT = new File(ROOT_STR);

    JSONServiceProvider svc;

    public void setUp() {
        svc = new JSONServiceProvider();
    }

    private InputStream getInputStream(String name) throws FileNotFoundException {
        File file = new File(ROOT, name);
        return new FileInputStream(file);
    }

   public void testXML() throws ParseException, IOException {
       String json = svc.fromXML(getInputStream("SIMPLE.xml"));
       Map m1 = svc.fromJSON(json);
       Assert.assertEquals("foo", m1.get("foo"));
       Assert.assertEquals(2, m1.size()); // Cannot have only one element !

       json = svc.fromXML(getInputStream("SIMPLE_2.xml"));
       Map m2 = svc.fromJSON(json);
       Assert.assertEquals("foo", m2.get("foo"));
       Assert.assertEquals("bar", m2.get("bar"));
       Assert.assertEquals(2, m2.size());

       json = svc.fromXML(getInputStream("EMPTY.json")); // empty file
       Map m4 = svc.fromJSON(json);
       Assert.assertEquals(0, m4.size());

       json = svc.fromXML(getInputStream("SIMPLE_3.xml")); // empty file
       Map m6 = svc.fromJSON(json);
       Assert.assertEquals(new Integer(1), m6.get("one"));
       Assert.assertEquals(new Double(2.4), m6.get("double"));
       Assert.assertEquals(new Boolean(true), m6.get("true"));
       Assert.assertEquals(new Boolean(false), m6.get("false"));
       Assert.assertEquals("foo", m6.get("foo"));
       Assert.assertEquals(svc.getNull(), m6.get("nil"));
       Assert.assertFalse(null == m6.get("nil")); // It's not null it's NULL (Javascript) !
       Map m62 = (Map) m6.get("object");
       Assert.assertEquals("foo", m62.get("foo"));
       Assert.assertEquals("bar", m62.get("bar"));
       Assert.assertEquals(2, m62.size());

   }

   public void testFromXMLWithArray() throws ParseException, FileNotFoundException, IOException {
       String json = svc.fromXML(getInputStream("COLLECTION.xml"));
       Map m1 = svc.fromJSON(json);
       Assert.assertEquals(2, m1.size()); // Cannot be alone ?
       List list = (List) m1.get("items");
       Assert.assertEquals(3, list.size());
       Map m11 = (Map) list.get(0);
       Assert.assertEquals(new Integer(1), m11.get("x"));
       Assert.assertEquals(new Integer(1), m11.get("y"));
   }

   public void testErrorInSimpleObject() {
       try {
           svc.fromXML("<xx>xx</xx> <y.y>2<y.y>,2}");
       } catch (ParseException e) {
           // OK
           return;
       }
       fail("ParseException expected");
   }

   public void testErrorInArray() {
       try {
           svc.fromXML("<xx><foo>foo</foo></xx><xx><bar>bar</xx>");
       } catch (ParseException e) {
           // OK
           return;
       }
       fail("ParseException expected");
   }

}
